@@ -9,18 +9,27 @@ import pytest
 from qtpy.QtCore import Qt
 
 from magicgui import magicgui, register_type, type_map, widgets
-from magicgui.signature import MagicSignature
+from magicgui.signature import MagicSignature, magic_signature
+
+
+def func(a: str = "works", b: int = 3, c=7.1) -> str:
+    return a + str(b)
 
 
 @pytest.fixture
 def magic_func():
     """Test function decorated by magicgui."""
+    return magicgui(func, call_button="my_button", auto_call=True, labels=False)
 
-    @magicgui(call_button="my_button", auto_call=True, labels=False)
-    def func(a: str = "works", b: int = 3, c=7.1) -> str:
-        return a + str(b)
 
-    return func
+@pytest.fixture
+def magic_func_defaults():
+    return magicgui(func)
+
+
+@pytest.fixture
+def magic_func_autocall():
+    return magicgui(func, auto_call=True)
 
 
 def test_magicgui(magic_func):
@@ -48,6 +57,16 @@ def test_magicgui(magic_func):
         magic_func.index(a)
 
 
+def test_default_call_button_behavior(magic_func_defaults, magic_func_autocall):
+    assert magic_func_defaults._call_button is not None
+
+    assert magic_func_autocall._call_button is None
+    prior_autocall_count = magic_func_autocall.call_count
+    magic_func_autocall.a.value = "hello"
+    magic_func_autocall.b.value = 7
+    assert magic_func_autocall.call_count == prior_autocall_count + 2
+
+
 def test_overriding_widget_type():
     """Test overriding the widget type of a parameter."""
     # a will now be a LineEdit instead of a spinbox
@@ -59,6 +78,24 @@ def test_overriding_widget_type():
     assert func.a.value == "1"
 
 
+def test_unrecognized_types():
+    """Test that arg with an unrecognized type is hidden."""
+
+    class Something:
+        pass
+
+    # don't know how to handle Something type
+    @magicgui
+    def func(arg: Something, b: int = 1):
+        pass
+
+    assert isinstance(func.arg, widgets.EmptyWidget)
+
+    with pytest.raises(TypeError) as e:
+        func()
+    assert "missing a required argument" in str(e)
+
+
 def test_no_type_provided():
     """Test position args with unknown type."""
 
@@ -66,7 +103,25 @@ def test_no_type_provided():
     def func(a):
         pass
 
-    assert isinstance(func.a, widgets.LiteralEvalLineEdit)
+    assert isinstance(func.a, widgets.EmptyWidget)
+    with pytest.raises(TypeError) as e:
+        func()
+    assert "missing a required argument" in str(e)
+    assert "@magicgui(a={'bind': value})" in str(e)
+
+
+def test_bind_out_of_order():
+    """Test that binding a value before a non-default argument still gives message."""
+
+    @magicgui(a={"bind": 10})
+    def func(a, x):
+        pass
+
+    assert isinstance(func.a, widgets.EmptyWidget)
+    with pytest.raises(TypeError) as e:
+        func()
+    assert "missing a required argument" in str(e)
+    assert "@magicgui(x={'bind': value})" in str(e)
 
 
 def test_call_button():
@@ -81,19 +136,20 @@ def test_call_button():
     func.a.value = 7
 
 
+@pytest.mark.filterwarnings("ignore")
 def test_auto_call(qtbot, magic_func):
     """Test that changing a parameter calls the function."""
+    from qtpy.QtTest import QTest
 
     # TODO: remove qtbot requirement so we can test other backends eventually.
-
     # changing the widget parameter calls the function
     with qtbot.waitSignal(magic_func.called, timeout=1000):
         magic_func.b.value = 6
 
     # changing the gui calls the function
     with qtbot.waitSignal(magic_func.called, timeout=1000):
-        qtbot.keyClick(magic_func.a.native, Qt.Key_A, Qt.ControlModifier)
-        qtbot.keyClick(magic_func.a.native, Qt.Key_Delete)
+        QTest.keyClick(magic_func.a.native, Qt.Key_A, Qt.ControlModifier)
+        QTest.keyClick(magic_func.a.native, Qt.Key_Delete)
 
 
 def test_dropdown_list_from_enum():
@@ -226,6 +282,7 @@ def test_invisible_param():
         return "works"
 
     assert hasattr(func, "a")
+    func.show()
     assert not func.a.visible
     assert func.b.visible
     assert func.c.visible
@@ -236,7 +293,7 @@ def test_bad_options():
     """Test that invalid parameter options raise TypeError."""
     with pytest.raises(TypeError):
 
-        @magicgui(b=7)
+        @magicgui(b=7)  # type: ignore
         def func(a="string", b=3, c=7.1):
             return "works"
 
@@ -264,26 +321,6 @@ def test_signature_repr():
     )
 
 
-def test_unrecognized_types():
-    """Test error handling when an arg with an unrecognized type is encountered."""
-
-    class Something:
-        pass
-
-    with pytest.raises(ValueError):
-        # don't know how to handle Something type
-        @magicgui
-        def func(arg: Something, b: int = 1):
-            pass
-
-    # # now it should not raise an error... but `arg` should not be in the gui
-    # core.SKIP_UNRECOGNIZED_TYPES = True
-    # with pytest.warns(UserWarning):
-    #     gui = func.Gui()
-    # assert not hasattr(gui, "arg")
-    # assert hasattr(gui, "b")
-
-
 def test_set_choices_raises():
     """Test failures on setting choices."""
 
@@ -310,22 +347,6 @@ def test_get_choices_raises():
     assert func.mood.choices == (1, 2, 3)
 
 
-@pytest.mark.skip(reason="does not yet work")
-def test_positions():
-    """Test that providing position options puts widget in the right place."""
-
-    def func(a=1, b=2, c=3):
-        pass
-
-    def get_layout_items(layout):
-        return [layout.itemAt(i).widget().objectName() for i in range(layout.count())]
-
-    gui = magicgui(func)
-    assert get_layout_items(gui.layout()) == ["a", "b", "c"]
-    gui = magicgui(func, a={"position": 2}, b={"position": 1}).Gui()
-    assert get_layout_items(gui.layout()) == ["c", "b", "a"]
-
-
 @pytest.mark.parametrize(
     "labels",
     [
@@ -350,9 +371,9 @@ def test_add_at_position(labels):
         return items
 
     gui = magicgui(func, labels=labels)
-    assert get_layout_items(gui) == ["a", "b", "c"]
+    assert get_layout_items(gui) == ["a", "b", "c", "call_button"]
     gui.insert(1, widgets.create_widget(name="new"))
-    assert get_layout_items(gui) == ["a", "new", "b", "c"]
+    assert get_layout_items(gui) == ["a", "new", "b", "c", "call_button"]
 
 
 def test_original_function_works(magic_func):
@@ -392,7 +413,7 @@ def test_register_types():
     class Sub2(Main2):
         pass
 
-    register_type(Main, choices=[1, 2, 3])
+    register_type(Main, choices=[None, 1, 2, 3])
     register_type(Main2, widget_type="LineEdit")
 
     @magicgui
@@ -451,7 +472,7 @@ def test_function_binding():
             self.name = name
             self.counter = 0.0
 
-        @magicgui(auto_call=True)
+        @magicgui(call_button="callme", sigma={"max": 365})
         def method(self, sigma: float = 1):
             self.counter = self.counter + sigma
             return self.name, self.counter
@@ -459,6 +480,8 @@ def test_function_binding():
     a = MyObject("a")
     b = MyObject("b")
 
+    assert a.method._call_button.text == "callme"  # type: ignore
+    assert a.method.sigma.max == 365
     assert a.method() == ("a", 1)
     assert b.method(sigma=4) == ("b", 4)
     assert a.method() == ("a", 2)
@@ -478,3 +501,228 @@ def test_call_count():
     assert func.call_count == 2
     func.reset_call_count()
     assert func.call_count == 0
+
+
+def test_tooltips_from_numpydoc():
+    """Test that numpydocs docstrings can be used for tooltips."""
+
+    x_tooltip = "override tooltip"
+    y_docstring = """A greeting, by default 'hi'. Notice how we miraculously pull
+the entirety of the docstring just like that"""
+
+    @magicgui(x={"tooltip": x_tooltip}, z={"tooltip": None})
+    def func(x: int, y: str = "hi", z=None):
+        """Do a little thing.
+
+        Parameters
+        ----------
+        x : int
+            An integer for you to use
+        y : str, optional
+            A greeting, by default 'hi'. Notice how we miraculously pull
+            the entirety of the docstring just like that
+        z : Any, optional
+            No tooltip for me please.
+        """
+
+    assert func.x.tooltip == x_tooltip
+    assert func.y.tooltip == y_docstring
+    assert not func.z.tooltip
+
+
+def test_duplicated_and_missing_params_from_numpydoc():
+    """Test that numpydocs docstrings can be used for tooltips."""
+
+    @magicgui
+    def func(x, y, z=None):
+        """Do a little thing.
+
+        Parameters
+        ----------
+        x, y : int
+            Integers for you to use
+        """
+
+    assert func.x.tooltip == "Integers for you to use"
+    assert func.y.tooltip == "Integers for you to use"
+    assert not func.z.tooltip
+
+
+def test_tooltips_from_google_doc():
+    """Test that google docstrings can be used for tooltips."""
+
+    x_docstring = "An integer for you to use"
+    y_docstring = """A greeting. Notice how we miraculously pull
+the entirety of the docstring just like that"""
+
+    @magicgui
+    def func(x: int, y: str = "hi"):
+        """Do a little thing.
+
+        Args:
+            x (int): An integer for you to use
+            y (str, optional): A greeting. Notice how we miraculously pull
+                               the entirety of the docstring just like that
+        """
+
+    assert func.x.tooltip == x_docstring
+    assert func.y.tooltip == y_docstring
+
+
+def test_tooltips_from_rest_doc():
+    """Test that google docstrings can be used for tooltips."""
+
+    x_docstring = "An integer for you to use"
+    y_docstring = """A greeting, by default 'hi'. Notice how we miraculously pull
+the entirety of the docstring just like that"""
+
+    @magicgui
+    def func(x: int, y: str = "hi", z=None):
+        """Do a little thing.
+
+        :param x: An integer for you to use
+        :param y: A greeting, by default 'hi'. Notice how we miraculously pull
+                  the entirety of the docstring just like that
+        :type x: int
+        :type y: str
+        """
+
+    assert func.x.tooltip == x_docstring
+    assert func.y.tooltip == y_docstring
+
+
+def test_no_tooltips_from_numpydoc():
+    """Test that ``tooltips=False`` hides all tooltips."""
+
+    @magicgui(tooltips=False)
+    def func(x: int, y: str = "hi"):
+        """Do a little thing.
+
+        Parameters
+        ----------
+        x : int
+            An integer for you to use
+        y : str, optional
+            A greeting, by default 'hi'
+        """
+
+    assert not func.x.tooltip
+    assert not func.y.tooltip
+
+
+def test_only_some_tooltips_from_numpydoc():
+    """Test that we can still show some tooltips with ``tooltips=False``."""
+    # tooltips=False, means docstrings wont be parsed at all, but tooltips
+    # can still be manually provided.
+    @magicgui(tooltips=False, y={"tooltip": "Still want a tooltip"})
+    def func(x: int, y: str = "hi"):
+        """Do a little thing.
+
+        Parameters
+        ----------
+        x : int
+            An integer for you to use
+        y : str, optional
+            A greeting, by default 'hi'
+        """
+
+    assert not func.x.tooltip
+    assert func.y.tooltip == "Still want a tooltip"
+
+
+def test_magicgui_type_error():
+
+    with pytest.raises(TypeError):
+        magicgui("not a function")  # type: ignore
+
+
+@magicgui
+def self_referencing_function(x: int = 1):
+    """Function that refers to itself, and wants the FunctionGui instance."""
+    return self_referencing_function
+
+
+def test_magicgui_self_reference():
+    """Test that self-referential magicguis work in global scopes."""
+
+    assert isinstance(self_referencing_function(), widgets.FunctionGui)
+
+
+def test_local_magicgui_self_reference():
+    """Test that self-referential magicguis work in local scopes."""
+
+    @magicgui
+    def local_self_referencing_function(x: int = 1):
+        """Function that refers to itself, and wants the FunctionGui instance."""
+        return local_self_referencing_function
+
+    assert isinstance(local_self_referencing_function(), widgets.FunctionGui)
+
+
+def test_empty_function():
+    """Test that a function with no params works."""
+
+    @magicgui(call_button=True)
+    def f():
+        ...
+
+    f.show()
+
+
+def test_boolean_label():
+    """Test that label can be used to set the text of a button widget."""
+
+    @magicgui(check={"label": "ABC"})
+    def test(check: bool, x=1):
+        pass
+
+    assert test.check.text == "ABC"
+
+    with pytest.warns(UserWarning) as record:
+
+        @magicgui(check={"text": "ABC", "label": "BCD"})
+        def test2(check: bool, x=1):
+            pass
+
+    assert "'text' and 'label' are synonymous for button widgets" in str(record[0])
+
+
+def test_none_defaults():
+    """Make sure that an unannotated parameter with default=None is ok."""
+    assert widgets.create_widget(value=None).value is None
+
+    def func(arg=None):
+        return 1
+
+    assert magicgui(func)() == 1
+
+    assert str(magic_signature(func)) == str(magicgui(func).__signature__)
+
+
+def test_update_and_dict():
+    @magicgui
+    def test(a: int = 1, y: str = "a"):
+        ...
+
+    assert test.asdict() == {"a": 1, "y": "a"}
+
+    test.update(a=10, y="b")
+    assert test.asdict() == {"a": 10, "y": "b"}
+
+    test.update({"a": 1, "y": "a"})
+    assert test.asdict() == {"a": 1, "y": "a"}
+
+    test.update([("a", 10), ("y", "b")])
+    assert test.asdict() == {"a": 10, "y": "b"}
+
+
+def test_update_on_call():
+    @magicgui
+    def test(a: int = 1, y: str = "a"):
+        ...
+
+    assert test.call_count == 0
+    test(a=10, y="b", update_widget=True)
+    assert test.a.value == 10
+    assert test.y.value == "b"
+    assert test.call_count == 1
